$call gdxxrw HSS_Model.xls o=Temp.gdx index=Index!a1

*Primary sets
SETS
TradeDate(*)                        'Trading Date'
TradePeriod(*)                      'Trading Period'
CalendarMonth(*)                    'Set of Calendar Month mmm-yy according to demand data'
Season(*)                           'Rating Season, used to select correct branch rating'
Node(*)                             'GXP'
Branch(*)                           'Branch ID'
RatingArea(*)                       'Branch Rating Area: HVDC, NORTH, SOUTH'
Island(*)                           'North Island and South Island'
BranchGroupConstraint(*)            'Branch group constraint name'
BranchGroupConstraintComponent(*)   'Branch group constraint components'
Scheme(*)                           'Generation scheme'
Offer(*)                            'Offer ID'

TradeBlock                          'Offer tranches' /t1, t2, t3, t4, t5/
MonthID(*)                          'Numbers represent month' /1*12/
DayID(*)                            'Numbers represent day of a month' /1*31/
WeekDayID(*)                        'Numbers represent day of a week' /1*7/
LossSegment(*)                      'Number of loss segment' /ls1*ls10/
FlowDirection                       'Flow direction F: Forward, B:Backward' /'Forward','Backward'/
;


Alias (Node,NodeFr,NodeTo), (TradeDate,StartDate,EndDate);
*Subsets and multidimensional sets
SETS
BranchNodeFr(Branch,Node)                                'Mapping Branch and Node From'
BranchNodeTo(Branch,Node)                                'Mapping Branch and Node To'
BranchRatingArea(Branch,RatingArea)                      'Rating area of a branch'
SchemeOffer(Scheme,Offer)                                'Shceme and Offer mapping'
OfferNode(Offer,Node)                                    'Offer and Node mapping'
TradeDateMapping(TradeDate,CalendarMonth,MonthID,DayID,WeekdayID) 'TradeDate Mapping'
;

*Calculated sets
SETS
ACBranch(Branch)                                         'AC branches'
DCBranch(Branch)                                         'HVDC links'
DCDirection(Branch,FlowDirection)                        'Direction of HVDC link F:Northward, B: Southward'
NodeIsland(Node,Island)                                  'Mapping Node Island'
RefNode(Node)                                            'Reference Node'
TradeDateCalendarMonth(TradeDate,CalendarMonth)          'TradeDate and Calendar Month Mapping'
TradeDateMonthID(TradeDate,MonthID)                      'TradeDate and MonthID Mapping'
TradeDateDayID(TradeDate,DayID)                          'TradeDate and DayID Mapping'
TradeDateWeekdayID(TradeDate,WeekdayID)                  'TradeDate Mapping'
DayTime(TradePeriod)                                     'Defining daytime period'
TradePeriodSeason(TradeDate,TradePeriod,Season)          'Defining seasonal rating for tradeperiod'
;

SCALAR TiwaiLoad                                         'Tiwai constant load assumption';

PARAMETERS
*Branch parameters
BranchOpenStatus(Branch)                                 'Branch open status, 1 = Open'
BranchSusceptance(Branch)                                'Branch Susceptance - B'
BranchResistance(Branch)                                 'Branch Resistance - R'
BranchFixedLoss(Branch)                                  'Branch Fixed Loss'
BranchNumLossTranches(Branch)                            'Number of loss tranches - used to piecewise the loss curve'
BranchCapacity(Branch,Season)                            'Seasonal rating of a branch'

*Branch group constraint parameters
BranchGroupConstraintFactor(BranchGroupConstraint,Branch)                              'Branch security constraint factors (sensitivities)'
BranchGroupConstraintParameter(BranchGroupConstraint,BranchGroupConstraintComponent)   'Branch group constraint parameter'

*demand and supply data
SchemeType(Scheme)                                                   '1 = Profiled output, 2 = Thermal, 3 = TKU and RPO and MAN, 4 = Waikato, 5 = Waitaki and Clutha'
OfferCapacity(Offer)                                                 'generation Offer nominal capacity'
OfferDerating(Offer,TradeDate,TradeDate)                             'reduction in capacity because of planned outages'
MonthlyDemand(CalendarMonth,Island)                                  'Island monthly load forecast GWh'
NodePeriodRatio(MonthID,WeekDayID,TradePeriod,Node)                       'Ratio of GXP load to period load - for each island'
PeriodDayRatio(MonthID,WeekDayID,TradePeriod,Island)                      'Ratio of period load to day load - for each island'
WeekDayBaseDayRatio(MonthID,WeekDayID,Island)                        'Ratio of weekday load to baseday load - for each island'

*Calculated parameter
BranchClosedStatus(Branch)                               'Branch open status, 1 = Open'
LossSegmentMW(Branch,LossSegment,Season)                 'MW capacity of each loss segment'
LossSegmentFactor(Branch,LossSegment,Season)             'Loss factor of each loss segment'
ACBranchLossMW(Branch,LossSegment,Season)                'MW capacity of each loss segment'
ACBranchLossFactor(Branch,LossSegment,Season)            'Loss factor of each loss segment'
HVDCBreakPointMWLoss(Branch,LossSegment,Season)          'HVDC peice-wise loss segment break point'
HVDCBreakPointMWFlow(Branch,LossSegment,Season)          'HVDC peice-wise flow segment break point'
ValidLossSegment(Branch,LossSegment)                     'Flag valid loss segment'

BranchGroupConstraintSense(BranchGroupConstraint)                                      'Branch security constraint sense(-1:<=, 0:= 1:>=)'
BranchGroupConstraintLimit(BranchGroupConstraint)                                      'Branch security constraint limit'

BlockTimeLength                                                      'Duration of a block'
TotalBaseDay(CalendarMonth,Island)                                   'Converting number of days to number of basedays in a month to estimate baseday load for that month'
BaseDayDemand(CalendarMonth, Island)                                 'Baseday demand of a calendar month GWh'
TradeDateIslandDemand(TradeDate, Island)                             'Tradedate demand of a calendar month GWh'
TradePeriodIslandDemand(TradeDate,TradePeriod,Island)                'Island tradeperiod demand MW'
TradePeriodNodeDemand(TradeDate, TradePeriod, Node)                  'GXP demand of a block of a trading date MW'

TradeDateOfferDerating(TradeDate,Offer)                              'reduction in capacity by trade date'
OfferActualCapacity(TradeDate,TradePeriod,Offer)                     'Actual Offer Capacity taking into account planned outage, forced outage and other constraints'
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,TradeBlock)           'Offer MW'
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,TradeBlock)        'Offer Price'

NodePeriodLoadSumRatio(TradeDate,TradePeriod,Island)                 'Testing'
;

*Coefficient to piece-wise loss function calculation
SCALAR LossCoeff_A    /0.3101/;
SCALAR LossCoeff_C    /0.14495/;
SCALAR LossCoeff_D    /0.32247/;
SCALAR LossCoeff_E    /0.46742/;
SCALAR LossCoeff_F    /0.82247/;
SCALAR MaxFlowSegment    /10000/;

$GDXIN Temp.gdx
$LOAD TradeDate TradePeriod CalendarMonth Season
$LOAD Node Branch RatingArea Island
$LOAD BranchGroupConstraint BranchGroupConstraintComponent
$LOAD Scheme Offer
$LOAD BranchNodeFr BranchNodeTo BranchRatingArea
$LOAD SchemeOffer OfferNode TradeDateMapping
$LOAD TiwaiLoad
$LOAD BranchOpenStatus BranchSusceptance BranchResistance
$LOAD BranchFixedLoss BranchNumLossTranches BranchCapacity
$LOAD BranchGroupConstraintFactor BranchGroupConstraintParameter
$LOAD SchemeType OfferCapacity OfferDerating
$LOAD MonthlyDemand NodePeriodRatio PeriodDayRatio WeekDayBaseDayRatio
$GDXIN

**************************Grid parameters and sets calculation *************************************************************************************************************************

*Set reference node for each island
RefNode('HAY2201') = yes;
RefNode('BEN2201') = yes;

NodeIsland(Node,'NI') $ Sum[Branch $ BranchRatingArea(Branch,'NORTH'), BranchNodeFr(Branch,Node) + BranchNodeTo(Branch,Node)]=yes;
NodeIsland(Node,'SI') $ (not NodeIsland(Node,'NI')) = yes;

DCBranch(Branch) $  BranchRatingArea(Branch,'HVDC')= yes ;
ACBranch(Branch) $ (not DCBranch(Branch)) = yes;

DCDirection(DCBranch,FlowDirection) $ [ (ord(FlowDirection) = 1) and (sum[(Node,Island) $ [ BranchNodeTo(DCBranch,Node) and NodeIsland(Node,Island) and (ord(Island) =1) ], 1 ]) ]= yes;
DCDirection(DCBranch,FlowDirection) $ [ (ord(FlowDirection) = 2) and (sum[(Node,Island) $ [ BranchNodeTo(DCBranch,Node) and NodeIsland(Node,Island) and (ord(Island) =2) ], 1 ]) ]= yes;

BranchClosedStatus(Branch) $ (not BranchOpenStatus(Branch)) = yes;

BranchGroupConstraintSense(BranchGroupConstraint) = BranchGroupConstraintParameter(BranchGroupConstraint,'Sense');
BranchGroupConstraintLimit(BranchGroupConstraint) = BranchGroupConstraintParameter(BranchGroupConstraint,'Limit');

*Branch Loss Function - Use the external loss model as provided by Transpower

*AC Loss function
*Segment 1
LossSegmentMW(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (ord(LossSegment) = 1)] = BranchCapacity(ACBranch,Season);
LossSegmentMW(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (BranchNumLossTranches(ACBranch) > 1) and (ord(LossSegment) = 1)] = BranchCapacity(ACBranch,Season) * LossCoeff_A;
LossSegmentFactor(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (BranchNumLossTranches(ACBranch) > 1) and (ord(LossSegment) = 1)] = 0.01 * BranchResistance(ACBranch) * BranchCapacity(ACBranch,Season) * 0.75 * LossCoeff_A;
*Segment 2
LossSegmentMW(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (BranchNumLossTranches(ACBranch) > 1) and (ord(LossSegment) = 2)] = BranchCapacity(ACBranch,Season) * (1-LossCoeff_A);
LossSegmentFactor(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (BranchNumLossTranches(ACBranch) > 1) and (ord(LossSegment) = 2)] = 0.01 * BranchResistance(ACBranch) * BranchCapacity(ACBranch,Season);
*Segment 3
LossSegmentMW(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (BranchNumLossTranches(ACBranch) > 1) and (ord(LossSegment) = 3)] = MaxFlowSegment;
LossSegmentFactor(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (BranchNumLossTranches(ACBranch) > 1) and (ord(LossSegment) = 3)] = 0.01 * BranchResistance(ACBranch) * BranchCapacity(ACBranch,Season) * (2 - (0.75*LossCoeff_A));

*Create branch loss segments
ACBranchLossMW(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (ord(LossSegment) = 1)] = LossSegmentMW(ACBranch,LossSegment,Season);
ACBranchLossMW(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (ord(LossSegment) > 1) and (ord(LossSegment) <= BranchNumLossTranches(ACBranch))] = LossSegmentMW(ACBranch,LossSegment,Season) - LossSegmentMW(ACBranch,LossSegment-1,Season);
ACBranchLossFactor(ACBranch,LossSegment,Season) $ [BranchClosedStatus(ACBranch) and (BranchNumLossTranches(ACBranch) > 1)] = LossSegmentFactor(ACBranch,LossSegment,Season);

ValidLossSegment(ACBranch,LossSegment) $ (ord(LossSegment) =1 ) = yes;
ValidLossSegment(ACBranch,LossSegment) $ [(Sum[Season,LossSegmentMW(ACBranch,LossSegment,Season)] > 0) and (ord(LossSegment) > 1 )] = yes;

*DC Loss function
*Segment 1
LossSegmentMW(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 1)] = BranchCapacity(DCBranch,Season) * LossCoeff_C;
LossSegmentFactor(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 1)] = 0.0001 * BranchResistance(DCBranch) * BranchCapacity(DCBranch,Season) * 0.75 * LossCoeff_C;
*Segment 2
LossSegmentMW(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 2)] = BranchCapacity(DCBranch,Season) * LossCoeff_D;
LossSegmentFactor(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 2)] = 0.0001 * BranchResistance(DCBranch) * BranchCapacity(DCBranch,Season) * LossCoeff_E;
*Segment 3
LossSegmentMW(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 3)] = BranchCapacity(DCBranch,Season) * 0.5;
LossSegmentFactor(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 3)] = 0.0001 * BranchResistance(DCBranch) * BranchCapacity(DCBranch,Season) * LossCoeff_F;
*Segment 4
LossSegmentMW(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 4)] = BranchCapacity(DCBranch,Season) * (1 - LossCoeff_D);
LossSegmentFactor(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 4)] = 0.0001 * BranchResistance(DCBranch) * BranchCapacity(DCBranch,Season) * (2 - LossCoeff_F);
*Segment 5
LossSegmentMW(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 5)] = BranchCapacity(DCBranch,Season) * (1 - LossCoeff_C);
LossSegmentFactor(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 5)] = 0.0001 * BranchResistance(DCBranch) * BranchCapacity(DCBranch,Season) * (2 - LossCoeff_E);
*Segment 6
LossSegmentMW(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 6)] = MaxFlowSegment;
LossSegmentFactor(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 6)] = 0.0001 * BranchResistance(DCBranch) * BranchCapacity(DCBranch,Season) * (2 - (0.75*LossCoeff_C));

*Let the first point on the HVDCBreakPointMWFlow and HVDCBreakPointMWLoss be 0
*This allows zero losses and zero flow on the HVDC links otherwise model could be infeasible
HVDCBreakPointMWFlow(DCBranch,LossSegment,Season) $ (ord(LossSegment) = 1) = 0;
HVDCBreakPointMWLoss(DCBranch,LossSegment,Season) $ (ord(LossSegment) = 1) = 0;

HVDCBreakPointMWFlow(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (ord(LossSegment) <= BranchNumLossTranches(DCBranch)+1) and (ord(LossSegment) > 1)] = LossSegmentMW(DCBranch,LossSegment-1,Season);
HVDCBreakPointMWLoss(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1) and (ord(LossSegment) = 2)] = (LossSegmentMW(DCBranch,LossSegment-1,Season) * LossSegmentFactor(DCBranch,LossSegment-1,Season));

loop((DCBranch,LossSegment) $ (ord(LossSegment) > 2 and (ord(LossSegment) <= BranchNumLossTranches(DCBranch)+1)),
      HVDCBreakPointMWLoss(DCBranch,LossSegment,Season) $ [BranchClosedStatus(DCBranch) and (BranchNumLossTranches(DCBranch) > 1)] = [(LossSegmentMW(DCBranch,LossSegment-1,Season) - LossSegmentMW(DCBranch,LossSegment-2,Season)) * LossSegmentFactor(DCBranch,LossSegment-1,Season)] + HVDCBreakPointMWLoss(DCBranch,LossSegment-1,Season);
    );

ValidLossSegment(DCBranch,LossSegment) $ [(Sum[Season,HVDCBreakPointMWFlow(DCBranch,LossSegment+1,Season)] > 0) and (ord(LossSegment) = 1)] = yes;
ValidLossSegment(DCBranch,LossSegment) $ [(Sum[Season,HVDCBreakPointMWFlow(DCBranch,LossSegment,Season)] > 0) and (ord(LossSegment) > 1)] = yes;

**************************Grid parameters and sets calculation end *********************************************************************************************************************


TradeDateCalendarMonth(TradeDate,CalendarMonth) = Sum[(MonthID,DayID,WeekDayID), TradeDateMapping(TradeDate,CalendarMonth,MonthID,DayID,WeekDayID)];
TradeDateMonthID(TradeDate,MonthID) = Sum[(CalendarMonth,DayID,WeekDayID), TradeDateMapping(TradeDate,CalendarMonth,MonthID,DayID,WeekDayID)];
TradeDateDayID(TradeDate,DayID) = Sum[(CalendarMonth,MonthID,WeekDayID), TradeDateMapping(TradeDate,CalendarMonth,MonthID,DayID,WeekDayID)];
TradeDateWeekDayID(TradeDate,WeekDayID) = Sum[(CalendarMonth,MonthID,DayID), TradeDateMapping(TradeDate,CalendarMonth,MonthID,DayID,WeekDayID)];


**************************Demand parameters and sets calculation ***********************************************************************************************************************

* Recalculate SI monthly demand excluding Tiwai Load
MonthlyDemand(CalendarMonth,'SI') = MonthlyDemand(CalendarMonth,'SI') - TiwaiLoad * 24 * Sum(TradeDate $ TradeDateCalendarMonth(TradeDate,CalendarMonth),1)/1000;

* Calculate the length of a TradePeriod (hrs)
BlockTimeLength = 24/card(TradePeriod);
Display BlockTimeLength;

* Calculate relative number of basedays in a month
TotalBaseDay(CalendarMonth,Island) =
Sum[(TradeDate,MonthID,WeekDayID) $ [TradeDateCalendarMonth(TradeDate,CalendarMonth) and TradeDateMonthID(TradeDate,MonthID) and TradeDateWeekDayID(TradeDate,WeekDayID)],
WeekDayBaseDayRatio(MonthID,WeekDayID,Island)];
Display TotalBaseDay, CalendarMonth, Island;

* Calculate Island Demand of a BaseDay of a month (GWh)
BaseDayDemand(CalendarMonth, Island) =  MonthlyDemand(CalendarMonth,Island) / TotalBaseDay(CalendarMonth,Island);
Display BaseDayDemand;

* Calculate Island Demand of a TradeDate of a month (GWh)
TradeDateIslandDemand(TradeDate, Island) =
Sum[(CalendarMonth,MonthID,WeekDayID) $ [TradeDateCalendarMonth(TradeDate,CalendarMonth) and TradeDateMonthID(TradeDate,MonthID) and TradeDateWeekDayID(TradeDate,WeekDayID)],
BaseDayDemand(CalendarMonth, Island) * WeekDayBaseDayRatio(MonthID,WeekDayID,Island)];

* Calculate Island Period Demand (MW)
TradePeriodIslandDemand(TradeDate,TradePeriod,Island) =
Sum[(MonthID,WeekDayID) $ [TradeDateMonthID(TradeDate,MonthID) and TradeDateWeekDayID(TradeDate,WeekDayID)],
TradeDateIslandDemand(TradeDate, Island) * PeriodDayRatio(MonthID,WeekDayID,TradePeriod,Island)*1000/BlockTimeLength ];

* Calculate GXP Demand (MW)
TradePeriodNodeDemand(TradeDate, TradePeriod, Node) =
Sum[(Island,MonthID,WeekDayID) $ [NodeIsland(Node,Island) and TradeDateMonthID(TradeDate,MonthID) and TradeDateWeekDayID(TradeDate,WeekDayID)],
TradePeriodIslandDemand(TradeDate,TradePeriod,Island) * NodePeriodRatio(MonthID,WeekDayID,TradePeriod,Node)];

TradePeriodNodeDemand(TradeDate, TradePeriod, 'TWI2201')  = TiwaiLoad;

* Testing
NodePeriodLoadSumRatio(TradeDate,TradePeriod,Island) = Sum[(Node,MonthID,WeekDayID) $ [NodeIsland(Node,Island) and TradeDateMonthID(TradeDate,MonthID) and TradeDateWeekDayID(TradeDate,WeekDayID)],
NodePeriodRatio(MonthID,WeekDayID,TradePeriod,Node)];
Display NodePeriodLoadSumRatio;

MonthlyDemand(CalendarMonth,Island) = Sum([TradeDate, TradePeriod, Node] $ NodeIsland(Node,Island) ,
TradePeriodNodeDemand(TradeDate, TradePeriod, Node) * TradeDateCalendarMonth(TradeDate,CalendarMonth) * BlockTimeLength/1000);
Display MonthlyDemand;

TradePeriodNodeDemand(TradeDate, TradePeriod, Node) = round(TradePeriodNodeDemand(TradeDate, TradePeriod, Node) ,3);

**************************Demand parameters and sets calculation end *******************************************************************************************************************


**************************Seasonal sets calculation ************************************************************************************************************************************

* Define daytime period to estimate offer capacity
DayTime(TradePeriod) $ [(ord(TradePeriod)*BlockTimeLength >= 7) and (ord(TradePeriod)*BlockTimeLength<= 21)] = yes;

* Defining TradePeriod Season
TradePeriodSeason(TradeDate,TradePeriod,'Winter') $ (not DayTime(TradePeriod)) = yes;
TradePeriodSeason(TradeDate,TradePeriod,'Winter') $ [DayTime(TradePeriod) and Sum(MonthID $ [(ord(MonthID)>5) and (ord(MonthID)<10)], TradeDateMonthID(TradeDate,MonthID))] = yes;
TradePeriodSeason(TradeDate,TradePeriod,'Winter') $ [DayTime(TradePeriod) and Sum([MonthID,DayID] $ [(ord(MonthID)=5) and (ord(DayID)>=10)], TradeDateMonthID(TradeDate,MonthID)*TradeDateDayID(TradeDate,DayID))] = yes;
TradePeriodSeason(TradeDate,TradePeriod,'Winter') $ [DayTime(TradePeriod) and Sum([MonthID,DayID] $ [(ord(MonthID)=10) and (ord(DayID)<=19)], TradeDateMonthID(TradeDate,MonthID)*TradeDateDayID(TradeDate,DayID))] = yes;
TradePeriodSeason(TradeDate,TradePeriod,'Winter') $ [DayTime(TradePeriod) and Sum([MonthID,DayID] $ [(ord(MonthID)=10) and (ord(DayID)<=19)], TradeDateMonthID(TradeDate,MonthID)*TradeDateDayID(TradeDate,DayID))] = yes;

TradePeriodSeason(TradeDate,TradePeriod,'Summer') $ [DayTime(TradePeriod) and Sum(MonthID $ [(ord(MonthID)>=12) or (ord(MonthID)<=2)], TradeDateMonthID(TradeDate,MonthID))] = yes;
TradePeriodSeason(TradeDate,TradePeriod,'Summer') $ [DayTime(TradePeriod) and Sum([MonthID,DayID] $ [(ord(MonthID)=3) and (ord(DayID)<15)], TradeDateMonthID(TradeDate,MonthID)*TradeDateDayID(TradeDate,DayID))] = yes;

TradePeriodSeason(TradeDate,TradePeriod,'Shoulder') = yes;
TradePeriodSeason(TradeDate,TradePeriod,'Shoulder') $ [TradePeriodSeason(TradeDate,TradePeriod,'Winter') or TradePeriodSeason(TradeDate,TradePeriod,'Summer')] = no;

**************************Seasonal sets calculation end ********************************************************************************************************************************


**************************Supply parameters and sets calculation ***********************************************************************************************************************

* Offer derating by tradedate calculation
TradeDateOfferDerating(TradeDate,Offer) = Sum[(StartDate,EndDate) $ [(ord(StartDate) <= ord(TradeDate)) and (ord(EndDate) >= ord(TradeDate))], OfferDerating(Offer,StartDate,EndDate)];

* Actual Capacity taking into account planned outages
OfferActualCapacity(TradeDate,TradePeriod,Offer) =  Max[0,OfferCapacity(Offer) - TradeDateOfferDerating(TradeDate,Offer)];

*Offer for profiled output scheme - SchemeType = 1
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ (Sum[Scheme $ SchemeOffer(Scheme,Offer), SchemeType(Scheme)] = 1) = OfferActualCapacity(TradeDate,TradePeriod, Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ (Sum[Scheme $ SchemeOffer(Scheme,Offer), SchemeType(Scheme)] = 1) = 0;

* HLY Maintenance Outage - one Huntly unitis assumes to be out of service for maintenance 3 weekends in 4 --> to make it simple, assume maximum 3 units are available during all weekends
OfferActualCapacity[TradeDate, TradePeriod, 'HLY2201 HLY0'] $ [Sum(WeekDayID $ [(ord(WeekDayID)=1) or (ord(WeekDayID)=7)],TradeDateWeekDayID(TradeDate,WeekDayID))]
= min(729,OfferActualCapacity[TradeDate, TradePeriod, 'HLY2201 HLY0']);

* Cooling water limits for Huntly 1-4
OfferActualCapacity[TradeDate, TradePeriod, 'HLY2201 HLY0'] $ [Sum(MonthID $ (ord(MonthID)<=3), TradeDateMonthID(TradeDate,MonthID))] =
min(729,OfferActualCapacity[TradeDate, TradePeriod, 'HLY2201 HLY0']);

OfferActualCapacity[TradeDate, DayTime, 'HLY2201 HLY0'] $ [Sum(MonthID $ [ord(MonthID)=2], TradeDateMonthID(TradeDate,MonthID))] =
min(722,OfferActualCapacity[TradeDate, DayTime, 'HLY2201 HLY0']);

* Otahuhu Capacity adjusted because of gas constraint
OfferActualCapacity[TradeDate, TradePeriod, 'OTA2202 OTC0'] = 360;
OfferActualCapacity[TradeDate, TradePeriod, 'OTA2202 OTC0'] $ [not DayTime(TradePeriod)] = 200;

* Otahuhu will be off during summer
*OfferActualCapacity[TradeDate, TradePeriod, 'OTA2202 OTC0'] $ [Sum(MonthID $ (ord(MonthID)>=10), TradeDateMonthID(TradeDate,MonthID))] = 0;
*OfferActualCapacity[TradeDate, TradePeriod, 'OTA2202 OTC0'] $ [Sum(MonthID $ (ord(MonthID)<=3), TradeDateMonthID(TradeDate,MonthID))] = 0;
*OfferActualCapacity[TradeDate, TradePeriod, 'OTA2202 OTC0'] $ [Sum(MonthID $ [(ord(MonthID)=1) or (ord(MonthID)=2)], TradeDateMonthID(TradeDate,MonthID))] = 0;

* TCC Capacity adjusted because of gas constraint
OfferActualCapacity[TradeDate, TradePeriod, 'SFD2201 SPL0'] = 360;
OfferActualCapacity[TradeDate, TradePeriod, 'SFD2201 SPL0'] $ [not DayTime(TradePeriod)] = 200;

*Straford Peaker will be off because there is not enough gas to run.
OfferActualCapacity[TradeDate, TradePeriod, 'SFD2201 SPL21'] = 0;
OfferActualCapacity[TradeDate, TradePeriod, 'SFD2201 SPL22'] = 0;

* All thermal are derated by a forced outage factor of 3%
OfferActualCapacity(TradeDate, TradePeriod, Offer)$(Sum[Scheme $ SchemeOffer(Scheme,Offer), SchemeType(Scheme)] = 1) = OfferActualCapacity(TradeDate,TradePeriod,Offer)*(1-0.03);

*Ancillary Services requirement
OfferActualCapacity[TradeDate,TradePeriod,'HLY2201 HLY0'] $ [not DayTime(TradePeriod)] = OfferActualCapacity[TradeDate,TradePeriod,'HLY2201 HLY0'] - 130;

*Offer for Thermal
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ (Sum[Scheme $ SchemeOffer(Scheme,Offer), SchemeType(Scheme)] = 2) = OfferActualCapacity(TradeDate,TradePeriod, Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ (Sum[Scheme $ SchemeOffer(Scheme,Offer), SchemeType(Scheme)] = 2) = 1;

*TN (testing)
TradePeriodOfferPrice(TradeDate,TradePeriod,'OTA2202 OTC0','t1') = 3;
TradePeriodOfferPrice(TradeDate,TradePeriod,'SFD2201 SPL0','t1') = 3;

*Offer for Rangipo
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Rangipo',Offer) = OfferActualCapacity(TradeDate,TradePeriod,Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Rangipo',Offer) = 5;

*Offer for Tokaanu
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Tokaanu',Offer) = OfferActualCapacity(TradeDate,TradePeriod,Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Tokaanu',Offer) = 5;

*Offer for Waikato
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Waikato',Offer) = OfferActualCapacity(TradeDate,TradePeriod,Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Waikato',Offer) = 20;

*Offer for Waitaki
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Waitaki',Offer) = OfferActualCapacity(TradeDate,TradePeriod,Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Waitaki',Offer) = 40;

*Offer for Clutha
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Clutha',Offer) = OfferActualCapacity(TradeDate,TradePeriod,Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Clutha',Offer) = 40;

*Offer for Manapouri
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Manapouri',Offer) = OfferActualCapacity(TradeDate,TradePeriod,Offer);
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,'t1') $ SchemeOffer('Manapouri',Offer) = 10;

**************************Demand and supply parameters and sets calculation end ********************************************************************************************************

execute_unload 'Inputs\HSS_Data.gdx', TradeDate, TradePeriod, MonthID, DayID, WeekDayID, CalendarMonth, Season, BlockTimeLength
      TradeDateCalendarMonth, TradeDateMonthID, TradeDateDayID, TradeDateWeekDayID, DayTime, TradePeriodSeason

      Node, Branch, BranchNodeFr, BranchNodeTo, Island, LossSegment, FlowDirection
      BranchCapacity, BranchSusceptance, BranchResistance, BranchFixedLoss, BranchNumLossTranches
      NodeIsland, DCBranch, ACBranch, RefNode, BranchOpenStatus, BranchClosedStatus
      ACBranchLossMW, ACBranchLossFactor, HVDCBreakPointMWFlow, HVDCBreakPointMWLoss, DCDirection, ValidLossSegment
      BranchGroupConstraint, BranchGroupConstraintFactor, BranchGroupConstraintSense, BranchGroupConstraintLimit

      Offer, TradeBlock, Scheme, SchemeType, SchemeOffer
      TradePeriodNodeDemand, OfferNode, TradePeriodOfferMW, TradePeriodOfferPrice;

*execute 'gams HSS_Run'

execute 'del *.gdx'
execute 'del *.put'
execute 'del *.lxi'
execute 'del *.lst'
