$call gams HSS_Model s=Model

execute 'gams HSS_Data'

SETS
*Period
TradeDate(*)                     'Dates definition'
TradePeriod(*)                   'Trading periods of a day'
Season(*)                        'Seasonal Rating'
MonthID(*)                       'Month ID'
DayID (*)                        'Day ID'
*Offer
Offer(*)                         'SPD Offer - MarketNode'
Scheme(*)                        'Trade group definition - used in calculation of total energy output GWh'

*Grid
Island(*)                        'Island definition'
Node(*)                          'SPD Node definitions'
Branch(*)                        'Branch ID'
BranchGroupConstraint(*)         'Transmission constraint definition'

*Mapping
SchemeOffer(Scheme,Offer)        'Mapping scheme and offer'
OfferNode(Offer,Node)            'Mapping offer and node'
NodeIsland(Node,Island)          'Mapping Node and Island'
BranchNodeFr(Branch,Node)        'Sending Node Definition'
BranchNodeTo(Branch,Node)        'Receiving Node Definition'

TradeDateMonthID(TradeDate,MonthID)       'TradeDate MonthID and DayID Mapping'
TradeDateDayID(TradeDate,DayID)           'TradeDate MonthID and DayID Mapping'

TradePeriodSeason(TradeDate,TradePeriod,Season) 'Seasonal rating applied for a trade period'
;

Alias (FromNode,Node), (ToNode,Node), (Offer,Offer1);
*Output
PARAMETERS
SchemeType(Scheme)                                       'Type of a Scheme: 1 = Profiled output, 2 = Thermal, 3 = TKU and RPO and MAN, 4 = Waikato, 5 = Waitaki and Clutha'

*Static Branch Data
BranchCapacity(Branch,Season)                            'Capacity of a branch'
BranchOpenStatus(Branch)                                 'Branch open status, 1 = Open'
BranchClosedStatus(Branch)                               'Branch open status, 1 = Open'

*Static Branch constraint data
BranchGroupConstraintSense(BranchGroupConstraint)                  'Branch security constraint sense for the current trading period (-1:<=, 0:= 1:>=)'
BranchGroupConstraintLimit(BranchGroupConstraint)                  'Branch security constraint limit for the current trading period'

*Trading period demand and offer data
TradePeriodNodeDemand(TradeDate,TradePeriod,Node)        'Demand data'

*Output Data
TradePeriodOfferCleared(TradeDate,TradePeriod,Offer)    'Output offer cleared by trading period'
TradePeriodNodePrice(TradeDate,TradePeriod,Node)        'Output node marginal price'
TradePeriodBranchFlow(TradeDate,TradePeriod,Branch)     'Output branch flow'
TradePeriodBranchMarginal(TradeDate,TradePeriod,Branch) 'Output branch marginal price'
TradePeriodBranchLoss(TradeDate,TradePeriod,Branch)     'Output branch loss'
TradePeriodBranchFixedLoss(TradeDate,TradePeriod,Branch)'Output branch loss'
TradePeriodBranchConstraintLHS(TradeDate,TradePeriod,BranchGroupConstraint)     'Output branch constraint LHS value'
TradePeriodBranchConstraintMarginal(TradeDate,TradePeriod,BranchGroupConstraint)          'Output branch constraint marginal value'

oTradePeriodOfferCleared(TradeDate,TradePeriod,Offer)    'Output offer cleared by trading period'
oTradePeriodNodePrice(TradeDate,TradePeriod,Node)        'Output node marginal price'
oTradePeriodBranchFlow(TradeDate,TradePeriod,Branch)     'Output branch flow'
oTradePeriodBranchMarginal(TradeDate,TradePeriod,Branch) 'Output branch marginal price'
oTradePeriodBranchLoss(TradeDate,TradePeriod,Branch)     'Output branch loss'
oTradePeriodBranchFixedLoss(TradeDate,TradePeriod,Branch)'Output branch loss'
oTradePeriodBranchConstraintLHS(TradeDate,TradePeriod,BranchGroupConstraint)     'Output branch constraint LHS value'
oTradePeriodBranchConstraintMarginal(TradeDate,TradePeriod,BranchGroupConstraint)          'Output branch constraint marginal value'
;

SCALAR iter;

$GDXIN Inputs\HSS_Data.gdx
$LOAD Island Node Season Branch BranchGroupConstraint
$LOAD NodeIsland BranchNodeFr BranchNodeTo
$LOAD BranchCapacity BranchOpenStatus BranchClosedStatus
$LOAD BranchGroupConstraintSense BranchGroupConstraintLimit
$LOAD TradeDate TradePeriod MonthID DayID
$LOAD TradeDateMonthID TradeDateDayID TradePeriodSeason
$LOAD Offer Scheme
$LOAD OfferNode SchemeOffer
$LOAD SchemeType
$LOAD TradePeriodNodeDemand
$GDXIN

FILE ModelDate                'File that define the date to run for each iteration';

Loop (TradeDate,
   put ModelDate;
   put "SET TradeDate(*)                     'Dates definition' /"TradeDate.tl:0"/; " /;
   putclose;

   put_utility 'Log' / "Solving TradeDate: " TradeDate.tl:0 /;
   put_utility 'exec' / 'gams HSS_Solve r=Model lo=3 ide=1';

*This is for testing
*   iter = ord(TradeDate);
*   if ((iter <= 10),  put_utility 'exec' / 'gams HSS_Solve r=Model lo=3 ide=1');

   execute_load 'Output' oTradePeriodOfferCleared, oTradePeriodNodePrice
                         oTradePeriodBranchFlow, oTradePeriodBranchMarginal
                         oTradePeriodBranchLoss, oTradePeriodBranchFixedLoss
                         oTradePeriodBranchConstraintLHS, oTradePeriodBranchConstraintMarginal;

   TradePeriodOfferCleared(TradeDate,TradePeriod,Offer) = oTradePeriodOfferCleared(TradeDate,TradePeriod,Offer);
   TradePeriodNodePrice(TradeDate,TradePeriod,Node) = oTradePeriodNodePrice(TradeDate,TradePeriod,Node);
   TradePeriodBranchFlow(TradeDate,TradePeriod,Branch) = oTradePeriodBranchFlow(TradeDate,TradePeriod,Branch);
   TradePeriodBranchMarginal(TradeDate,TradePeriod,Branch) = oTradePeriodBranchMarginal(TradeDate,TradePeriod,Branch);
   TradePeriodBranchLoss(TradeDate,TradePeriod,Branch) = oTradePeriodBranchLoss(TradeDate,TradePeriod,Branch);
   TradePeriodBranchFixedLoss(TradeDate,TradePeriod,Branch) = oTradePeriodBranchFixedLoss(TradeDate,TradePeriod,Branch);
   TradePeriodBranchConstraintLHS(TradeDate,TradePeriod,BranchGroupConstraint) = oTradePeriodBranchConstraintLHS(TradeDate,TradePeriod,BranchGroupConstraint);
   TradePeriodBranchConstraintMarginal(TradeDate,TradePeriod,BranchGroupConstraint) = oTradePeriodBranchConstraintMarginal(TradeDate,TradePeriod,BranchGroupConstraint);
);


execute_unload 'Outputs\Output_All' Island, Node, Season, Branch, BranchGroupConstraint
                            NodeIsland, BranchNodeFr, BranchNodeTo
                            TradeDate, TradePeriod, MonthID, DayID
                            TradeDateMonthID, TradeDateDayID, TradePeriodSeason
                            Offer Scheme
                            OfferNode SchemeOffer
                            SchemeType
                            BranchCapacity BranchOpenStatus BranchClosedStatus
                            BranchGroupConstraintSense BranchGroupConstraintLimit
                            TradePeriodNodeDemand TradePeriodOfferCleared TradePeriodNodePrice
                            TradePeriodBranchFlow TradePeriodBranchMarginal
                            TradePeriodBranchLoss TradePeriodBranchFixedLoss
                            TradePeriodBranchConstraintLHS TradePeriodBranchConstraintMarginal
                            ;


$ontext
*$Include Mode.inc
Display Mode;

if (Mode=1,
   execute 'gams HSS_Output_Comit';
elseif Mode=2,
   execute 'gams HSS_Output_Comit_All';
elseif Mode=3,
   execute 'gams HSS_Output_Spectra';
elseif Mode=4,
   execute 'gams HSS_Output_Spectra_All';
);
$offtext

*execute 'gams HSS_Output'

execute 'del *.gdx'
execute 'del *.put'
execute 'del *.lxi'
execute 'del *.lst'


